from enum import Enum, auto


class StrEnum(str, Enum):
    def __new__(cls, value, *args, **kwargs):
        if not isinstance(value, (str, auto)):
            raise TypeError(
                f"Values of StrEnums must be strings: {value!r} is a {type(value)}"
            )
        return super().__new__(cls, value, *args, **kwargs)

    def __str__(self):
        return str(self.value)

    def _generate_next_value_(name, *_):
        return name.lower()


class AreaMethod(StrEnum):
    Ellipsoidal = auto()
    """Use an ellipsoidal model of the Earth for area calculations.

    This uses the geodesic measurement methods given by [Karney (2013)].

    ## Assumptions

    - Polygons are assumed to be wound in a counter-clockwise direction for the exterior
      ring and a clockwise direction for interior rings. This is the standard winding
      for geometries that follow the Simple Feature standard. Alternative windings may
      result in a negative area. See "Interpreting negative area values" below.
    - Polygons are assumed to be smaller than half the size of the earth. If you expect
      to be dealing with polygons larger than this, please use the `unsigned` methods.

    ## Units

    - return value: meter²

    ## Interpreting negative area values

    A negative value can mean one of two things:

    1. The winding of the polygon is in the clockwise direction (reverse winding). If
       this is the case, and you know the polygon is smaller than half the area of
       earth, you can take the absolute value of the reported area to get the correct
       area.
    2. The polygon is larger than half the planet. In this case, the returned area of
       the polygon is not correct. If you expect to be dealing with very large polygons,
       please use the `unsigned` methods.

    [Karney (2013)]:  https://arxiv.org/pdf/1109.4448.pdf
    """

    Euclidean = auto()
    """Calculate planar area."""

    Spherical = auto()
    """Use a spherical model of the Earth for area calculations.

    Calculate the geodesic area of a geometry on a sphere using the algorithm presented
    in _Some Algorithms for Polygons on a Sphere_ by [Chamberlain and Duquette (2007)].

    [Chamberlain and Duquette (2007)]: https://trs.jpl.nasa.gov/handle/2014/41271

    ## Units

    - return value: meter²
    """


class GeoParquetEncoding(StrEnum):
    """Options for geometry encoding in GeoParquet."""

    WKB = auto()
    """Use Well-Known Binary (WKB) encoding when writing GeoParquet files.
    """

    Native = auto()
    """Use native GeoArrow geometry types when writing GeoParquet files.

    Supported as of GeoParquet version 1.1.

    This option provides for better read and write performance and for inferring spatial
    partitioning from remote files. But it does not yet have widespread support.
    """


class LengthMethod(StrEnum):
    Ellipsoidal = auto()
    """Determine the length of a geometry on an ellipsoidal model of the earth.

    This uses the geodesic measurement methods given by [Karney (2013)]. As opposed to
    older methods like Vincenty, this method is accurate to a few nanometers and always
    converges.

    [Karney (2013)]:  https://arxiv.org/pdf/1109.4448.pdf
    """

    Euclidean = auto()
    """Determine the length of a geometry using planar calculations."""

    Haversine = auto()
    """Determine the length of a geometry using the [haversine formula].

    [haversine formula]: https://en.wikipedia.org/wiki/Haversine_formula

    *Note*: this implementation uses a mean earth radius of 6371.088 km, based on the
    [recommendation of the IUGG](ftp://athena.fsv.cvut.cz/ZFG/grs80-Moritz.pdf)
    """

    Vincenty = auto()
    """Determine the length of a geometry using [Vincenty's formulae].

    [Vincenty's formulae]: https://en.wikipedia.org/wiki/Vincenty%27s_formulae
    """


class SimplifyMethod(StrEnum):
    RDP = auto()
    """Use the [Ramer-Douglas-Peucker
    algorithm](https://en.wikipedia.org/wiki/Ramer-Douglas-Peucker_algorithm) for
    LineString simplificatino.

    Polygons are simplified by running the RDP algorithm on
    all their constituent rings. This may result in invalid Polygons, and has no
    guarantee of preserving topology.

    Multi* objects are simplified by simplifying all their constituent geometries
    individually.
    """

    VW = auto()
    """Use the
    [Visvalingam-Whyatt](http://www.tandfonline.com/doi/abs/10.1179/000870493786962263)
    algorithm for LineString simplification.

    See [here](https://bost.ocks.org/mike/simplify/) for a graphical explanation
    Polygons are simplified by running the algorithm on all their constituent rings.

    This may result in invalid Polygons, and has no guarantee of preserving topology.
    Multi* objects are simplified by simplifying all their constituent geometries
    individually.
    """

    VW_Preserve = auto()
    """Use a topology-preserving variant of
    the
    [Visvalingam-Whyatt](http://www.tandfonline.com/doi/abs/10.1179/000870493786962263)
    algorithm for LineString simplification.

    See [here](https://www.jasondavies.com/simplify/) for a graphical explanation.

    The topology-preserving algorithm uses an R* tree to efficiently find candidate line
    segments which are tested for intersection with a given triangle. If intersections
    are found, the previous point (i.e. the left component of the current triangle) is
    also removed, altering the geometry and removing the intersection.

    # Notes

    - It is possible for the simplification algorithm to displace a Polygon's interior
      ring outside its shell.
    - The algorithm does **not** guarantee a valid output geometry, especially on
      smaller geometries.
    - If removal of a point causes a self-intersection, but the geometry only has `n +
      1` points remaining (3 for a `LineString`, 5 for a `Polygon`), the point is
      retained and the simplification process ends. This is because there is no
      guarantee that removal of two points will remove the intersection, but removal of
      further points would leave too few points to form a valid geometry.
    - The tolerance used to remove a point is `epsilon`, in keeping with GEOS. JTS uses
      `epsilon ^ 2`
    """



import geoarrow.rust.core as gars
import geodatasets
import geopandas as gpd
import shapely.geometry
from affine import Affine

nybb_path = geodatasets.get_path("nybb")


def test_affine_function():
    gdf = gpd.read_file(nybb_path)
    # shapely_area = gdf.geometry.area
    assert isinstance(gdf, gpd.GeoDataFrame)

    table = gars.from_geopandas(gdf)
    geom = table.geometry

    xoff = 10
    yoff = 20

    affine = Affine.translation(xoff, yoff)
    translated = gars.affine_transform(geom, affine)
    orig_geom = geom.chunk(0)[0]
    translated_geom = translated.chunk(0)[0]

    first_coord = shapely.geometry.shape(orig_geom).geoms[0].exterior.coords[0]
    first_coord_translated = (
        shapely.geometry.shape(translated_geom).geoms[0].exterior.coords[0]
    )

    assert first_coord_translated[0] - xoff == first_coord[0]
    assert first_coord_translated[1] - yoff == first_coord[1]



import geoarrow.rust.core as gars
import geodatasets
import geopandas as gpd
import numpy as np
import pyarrow as pa

nybb_path = geodatasets.get_path("nybb")


def test_area():
    gdf = gpd.read_file(nybb_path)
    shapely_area = gdf.geometry.area
    assert isinstance(gdf, gpd.GeoDataFrame)

    table = gars.from_geopandas(gdf)
    ga_area = table.geometry.area()
    assert ga_area.num_chunks() == 1

    pa_area = pa.array(ga_area.chunk(0))
    assert np.allclose(shapely_area, pa_area)



import geoarrow.rust.core as gars
import geodatasets
import geopandas as gpd
import numpy as np
import pytest
import shapely

nybb_path = geodatasets.get_path("nybb")


def test_from_shapely():
    gdf = gpd.read_file(nybb_path)
    shapely_orig = np.array(gdf.geometry)
    ga_arr = gars.from_shapely(shapely_orig)
    shapely_back = ga_arr.to_shapely()
    assert np.all(shapely_orig == shapely_back)
    assert isinstance(ga_arr, gars.MultiPolygonArray)
    ga_arr_back = gars.from_shapely(shapely_back)
    assert isinstance(ga_arr_back, gars.MultiPolygonArray)
    assert ga_arr == ga_arr_back


def test_from_shapely_chunked():
    gdf = gpd.read_file(nybb_path)
    shapely_orig = np.array(gdf.geometry)
    chunked_arr = gars.ChunkedMultiPolygonArray.from_shapely(shapely_orig, chunk_size=2)
    assert chunked_arr.num_chunks() == 3
    assert np.all(shapely_orig == chunked_arr.to_shapely())


@pytest.mark.xfail(
    reason=(
        "WKB -> Mixed array current parses as a mixed array, "
        " and the MultiPoint is never downcasted back to a Point."
    )
)
def test_from_shapely_mixed():
    point = shapely.points(1, 1)
    multi_polygon = gpd.read_file(nybb_path).geometry.iloc[0]
    mixed_shapely_geoms = np.array([point, multi_polygon])
    ga_arr = gars.from_shapely(mixed_shapely_geoms)
    assert isinstance(ga_arr, gars.MixedGeometryArray)
    assert isinstance(ga_arr[1], gars.MultiPolygon)

    shapely_back = ga_arr.to_shapely()
    assert shapely_back == ga_arr.to_shapely()



from __future__ import annotations

from pathlib import Path
from typing import (
    BinaryIO,
    Dict,
    List,
    Optional,
    Self,
    Sequence,
    Tuple,
    TypeVar,
    Union,
    overload,
)

try:
    import numpy as np
    from numpy.typing import NDArray
except ImportError:
    pass

try:
    import geopandas as gpd
except ImportError:
    pass

from .types import (
    AffineInputT,
    AffineTransform,
    ArrowArrayExportable,
    ArrowSchemaExportable,
    ArrowStreamExportable,
    AreaMethodT,
    BboxPaths,
    BroadcastGeometry,
    GeoInterfaceProtocol,
    GeoParquetEncodingT,
    IntFloat,
    LengthMethodT,
    NativeChunkedGeometryArrayT,
    NativeGeometryArrayT,
    NumpyArrayProtocolf64,
    SimplifyInputT,
    SimplifyMethodT,
)
from .enums import AreaMethod, GeoParquetEncoding, LengthMethod, SimplifyMethod

class Point:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class LineString:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class Polygon:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class MultiPoint:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class MultiLineString:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class MultiPolygon:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class Geometry:
    # def __arrow_c_array__(
    #     self, requested_schema: object | None = None
    # ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class GeometryCollection:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __repr__(self) -> str: ...
    def _repr_svg_(self) -> str: ...

class WKB:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __repr__(self) -> str: ...

class Rect:
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __repr__(self) -> str: ...

class PointArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> Point: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def convex_hull(self) -> PolygonArray: ...
    def envelope(self) -> RectArray: ...
    @classmethod
    def from_xy(
        cls,
        x: ArrowArrayExportable | NumpyArrayProtocolf64,
        y: ArrowArrayExportable | NumpyArrayProtocolf64,
    ) -> Self: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> Float64Array: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class LineStringArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> LineString: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def convex_hull(self) -> PolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> RectArray: ...
    def frechet_distance(self, other: BroadcastGeometry) -> Float64Array: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> Float64Array: ...
    def line_interpolate_point(
        self, fraction: float | int | ArrowArrayExportable | NumpyArrayProtocolf64
    ) -> PointArray: ...
    def line_locate_point(
        self, point: GeoInterfaceProtocol | ArrowArrayExportable
    ) -> Float64Array: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class PolygonArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> Polygon: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def convex_hull(self) -> PolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def polylabel(self, tolerance: float) -> PointArray: ...
    def is_empty(self) -> BooleanArray: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class MultiPointArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> MultiPoint: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def convex_hull(self) -> PolygonArray: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> Float64Array: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class MultiLineStringArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> MultiLineString: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def convex_hull(self) -> PolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> Float64Array: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class MultiPolygonArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> MultiPolygon: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def convex_hull(self) -> PolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class MixedGeometryArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> Geometry: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def convex_hull(self) -> PolygonArray: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_ewkb(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_wkt(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class GeometryCollectionArray:
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __getitem__(self, key: int) -> GeometryCollection: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def center(self) -> PointArray: ...
    def centroid(self) -> PointArray: ...
    def convex_hull(self) -> PolygonArray: ...
    def envelope(self) -> RectArray: ...
    def geodesic_perimeter(self) -> Float64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> Float64Array: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_ewkb(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    @classmethod
    def from_wkb(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_wkt(cls, input: ArrowArrayExportable) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def to_wkb(self) -> WKBArray: ...

class WKBArray:
    def __array__(self) -> NDArray[np.object_]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> WKB: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def __repr__(self) -> str: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...
    @classmethod
    def from_arrow(cls, input: ArrowArrayExportable) -> Self: ...
    @classmethod
    def from_shapely(cls, input) -> Self: ...
    def to_shapely(self) -> NDArray[np.object_]: ...

class RectArray:
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> Rect: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def to_polygon_array(self) -> PolygonArray: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class BooleanArray:
    def __array__(self) -> NDArray[np.bool_]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def to_numpy(self) -> NDArray[np.bool_]: ...

# class Float16Array:
#     def __array__(self) -> NDArray[np.float16]: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.float16]) -> Self: ...

# class Float32Array:
#     def __array__(self) -> NDArray[np.float32]: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def to_numpy(self) -> NDArray[np.float32]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.float32]) -> Self: ...

class Float64Array:
    def __array__(self) -> NDArray[np.float64]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __len__(self) -> int: ...
    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...
    def to_numpy(self) -> NDArray[np.float64]: ...
    @classmethod
    def from_numpy(cls, arr: NDArray[np.float64]) -> Self: ...

# class Int16Array:
#     def __array__(self) -> NDArray[np.int16]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.int16]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int16]) -> Self: ...

# class Int32Array:
#     def __array__(self) -> NDArray[np.int32]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.int32]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int32]) -> Self: ...

# class Int64Array:
#     def __array__(self) -> NDArray[np.int64]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.int64]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int64]) -> Self: ...

# class Int8Array:
#     def __array__(self) -> NDArray[np.int8]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.int8]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int8]) -> Self: ...

# class LargeStringArray:
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...

# class StringArray:
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...

# class UInt16Array:
#     def __array__(self) -> NDArray[np.uint16]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.uint16]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint16]) -> Self: ...

# class UInt32Array:
#     def __array__(self) -> NDArray[np.uint32]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.uint32]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint32]) -> Self: ...

# class UInt64Array:
#     def __array__(self) -> NDArray[np.uint64]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.uint64]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint64]) -> Self: ...

# class UInt8Array:
#     def __array__(self) -> NDArray[np.uint8]: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def __arrow_c_array__(
#         self, requested_schema: object | None = None
#     ) -> Tuple[object, object]: ...
#     def to_numpy(self) -> NDArray[np.uint8]: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint8]) -> Self: ...

class ChunkedPointArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> Point: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chunk(self, i: int) -> PointArray: ...
    def chunks(self) -> List[PointArray]: ...
    def concatenate(self) -> PointArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def num_chunks(self) -> int: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedLineStringArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> LineString: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def chunk(self, i: int) -> LineStringArray: ...
    def chunks(self) -> List[LineStringArray]: ...
    def concatenate(self) -> LineStringArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> ChunkedRectArray: ...
    def frechet_distance(self, other: BroadcastGeometry) -> ChunkedFloat64Array: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def line_interpolate_point(
        self, fraction: float | int | ArrowStreamExportable
    ) -> ChunkedPointArray: ...
    def line_locate_point(
        self, point: GeoInterfaceProtocol | ArrowStreamExportable
    ) -> ChunkedFloat64Array: ...
    def num_chunks(self) -> int: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedPolygonArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> Polygon: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def chunk(self, i: int) -> PolygonArray: ...
    def chunks(self) -> List[PolygonArray]: ...
    def concatenate(self) -> PolygonArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def num_chunks(self) -> int: ...
    def polylabel(self, tolerance: float) -> ChunkedPointArray: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedMultiPointArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> MultiPoint: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chunk(self, i: int) -> MultiPointArray: ...
    def chunks(self) -> List[MultiPointArray]: ...
    def concatenate(self) -> MultiPointArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def num_chunks(self) -> int: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedMultiLineStringArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> MultiLineString: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def chunk(self, i: int) -> MultiLineStringArray: ...
    def chunks(self) -> List[MultiLineStringArray]: ...
    def concatenate(self) -> MultiLineStringArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def length(
        self, *, method: LengthMethod | LengthMethodT = LengthMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def num_chunks(self) -> int: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedMultiPolygonArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> MultiPolygon: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chaikin_smoothing(self, n_iterations: int) -> Self: ...
    def chunk(self, i: int) -> MultiPolygonArray: ...
    def chunks(self) -> List[MultiPolygonArray]: ...
    def concatenate(self) -> MultiPolygonArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def densify(self, max_distance: float) -> Self: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def num_chunks(self) -> int: ...
    def simplify(
        self,
        epsilon: float,
        *,
        method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
    ) -> Self: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedMixedGeometryArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> Geometry: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chunk(self, i: int) -> MixedGeometryArray: ...
    def chunks(self) -> List[MixedGeometryArray]: ...
    def concatenate(self) -> MixedGeometryArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def num_chunks(self) -> int: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedGeometryCollectionArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> GeometryCollection: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def center(self) -> ChunkedPointArray: ...
    def centroid(self) -> ChunkedPointArray: ...
    def chunk(self, i: int) -> GeometryCollectionArray: ...
    def chunks(self) -> List[GeometryCollectionArray]: ...
    def concatenate(self) -> GeometryCollectionArray: ...
    def convex_hull(self) -> ChunkedPolygonArray: ...
    def envelope(self) -> ChunkedRectArray: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def geodesic_perimeter(self) -> ChunkedFloat64Array: ...
    def is_empty(self) -> BooleanArray: ...
    def num_chunks(self) -> int: ...
    def signed_area(
        self, *, method: AreaMethod | AreaMethodT = AreaMethod.Euclidean
    ) -> ChunkedFloat64Array: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedWKBArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __array__(self) -> NDArray[np.object_]: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> WKB: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def chunk(self, i: int) -> WKBArray: ...
    def chunks(self) -> List[WKBArray]: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    @classmethod
    def from_shapely(cls, input, *, chunk_size: int = 65536) -> Self: ...
    def num_chunks(self) -> int: ...
    def to_shapely(self) -> NDArray[np.object_]: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedRectArray:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __getitem__(self, key: int) -> Rect: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def chunk(self, i: int) -> RectArray: ...
    def chunks(self) -> List[RectArray]: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    def num_chunks(self) -> int: ...
    def total_bounds(self) -> Tuple[float, float, float, float]: ...

class ChunkedBooleanArray:
    def __array__(self) -> NDArray[np.bool_]: ...
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __len__(self) -> int: ...
    def chunk(self, i: int) -> BooleanArray: ...
    def chunks(self) -> List[BooleanArray]: ...
    @classmethod
    def from_arrow_arrays(cls, input: Sequence[ArrowArrayExportable]) -> Self: ...
    def num_chunks(self) -> int: ...
    def to_numpy(self) -> NDArray[np.bool_]: ...

# class ChunkedFloat16Array:
#     def __array__(self) -> NDArray[np.float16]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Float16Array: ...
#     def chunks(self) -> List[Float16Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.float16], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.float16]: ...

# class ChunkedFloat32Array:
#     def __array__(self) -> NDArray[np.float32]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Float32Array: ...
#     def chunks(self) -> List[Float32Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.float32], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.float32]: ...

class ChunkedFloat64Array:
    def __array__(self) -> NDArray[np.float64]: ...
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    def __len__(self) -> int: ...
    def chunk(self, i: int) -> Float64Array: ...
    def chunks(self) -> List[Float64Array]: ...
    def num_chunks(self) -> int: ...
    @classmethod
    def from_numpy(cls, arr: NDArray[np.float64], lengths: List[int]) -> Self: ...
    def to_numpy(self) -> NDArray[np.float64]: ...

# class ChunkedInt16Array:
#     def __array__(self) -> NDArray[np.int16]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Int16Array: ...
#     def chunks(self) -> List[Int16Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int16], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.int16]: ...

# class ChunkedInt32Array:
#     def __array__(self) -> NDArray[np.int32]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Int32Array: ...
#     def chunks(self) -> List[Int32Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int32], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.int32]: ...

# class ChunkedInt64Array:
#     def __array__(self) -> NDArray[np.int64]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Int64Array: ...
#     def chunks(self) -> List[Int64Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int64], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.int64]: ...

# class ChunkedInt8Array:
#     def __array__(self) -> NDArray[np.int8]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> Int8Array: ...
#     def chunks(self) -> List[Int8Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.int8], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.int8]: ...

# class ChunkedLargeStringArray:
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> LargeStringArray: ...
#     def chunks(self) -> List[LargeStringArray]: ...
#     def num_chunks(self) -> int: ...

# class ChunkedStringArray:
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> StringArray: ...
#     def chunks(self) -> List[StringArray]: ...
#     def num_chunks(self) -> int: ...

# class ChunkedUInt16Array:
#     def __array__(self) -> NDArray[np.uint16]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> UInt16Array: ...
#     def chunks(self) -> List[UInt16Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint16], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.uint16]: ...

# class ChunkedUInt32Array:
#     def __array__(self) -> NDArray[np.uint32]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> UInt32Array: ...
#     def chunks(self) -> List[UInt32Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint32], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.uint32]: ...

# class ChunkedUInt64Array:
#     def __array__(self) -> NDArray[np.uint64]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> UInt64Array: ...
#     def chunks(self) -> List[UInt64Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint64], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.uint64]: ...

# class ChunkedUInt8Array:
#     def __array__(self) -> NDArray[np.uint8]: ...
#     def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
#     def __eq__(self, other: Self) -> bool: ...
#     def __len__(self) -> int: ...
#     def chunk(self, i: int) -> UInt8Array: ...
#     def chunks(self) -> List[UInt8Array]: ...
#     def num_chunks(self) -> int: ...
#     @classmethod
#     def from_numpy(cls, arr: NDArray[np.uint8], lengths: List[int]) -> Self: ...
#     def to_numpy(self) -> NDArray[np.uint8]: ...

class GeoTable:
    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...
    def __eq__(self, other: Self) -> bool: ...
    @property
    def __geo_interface__(self) -> dict: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def explode(self) -> Self: ...
    @classmethod
    def from_arrow(cls, input: ArrowStreamExportable) -> Self: ...
    @classmethod
    def from_geopandas(cls, input: gpd.GeoDataFrame) -> Self: ...
    @property
    def geometry(
        self,
    ) -> (
        ChunkedPointArray
        | ChunkedLineStringArray
        | ChunkedPolygonArray
        | ChunkedMultiPointArray
        | ChunkedMultiLineStringArray
        | ChunkedMultiPolygonArray
        | ChunkedMixedGeometryArray
        | ChunkedGeometryCollectionArray
    ): ...
    @property
    def num_columns(self) -> int: ...
    def to_geopandas(self) -> gpd.GeoDataFrame: ...

# Top-level array/chunked array functions

@overload
def affine_transform(
    input: AffineInputT,
    transform: AffineTransform,
) -> AffineInputT: ...
@overload
def affine_transform(
    input: ArrowArrayExportable,
    transform: AffineTransform,
) -> NativeGeometryArrayT: ...
@overload
def affine_transform(
    input: ArrowStreamExportable,
    transform: AffineTransform,
) -> NativeChunkedGeometryArrayT: ...
def affine_transform(
    input: AffineInputT | ArrowArrayExportable | ArrowStreamExportable,
    transform: AffineTransform,
) -> AffineInputT | NativeGeometryArrayT | NativeChunkedGeometryArrayT: ...
@overload
def area(
    input: ArrowArrayExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> Float64Array: ...
@overload
def area(
    input: ArrowStreamExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> ChunkedFloat64Array: ...
def area(
    input: ArrowArrayExportable | ArrowStreamExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> Float64Array | ChunkedFloat64Array: ...
@overload
def center(input: ArrowArrayExportable) -> PointArray: ...
@overload
def center(input: ArrowStreamExportable) -> ChunkedPointArray: ...
def center(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> PointArray | ChunkedPointArray: ...
@overload
def centroid(input: ArrowArrayExportable) -> PointArray: ...
@overload
def centroid(input: ArrowStreamExportable) -> ChunkedPointArray: ...
def centroid(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> PointArray | ChunkedPointArray: ...

ChaikinSmoothingT = TypeVar(
    "ChaikinSmoothingT",
    LineStringArray,
    PolygonArray,
    MultiLineStringArray,
    MultiPolygonArray,
    ChunkedLineStringArray,
    ChunkedPolygonArray,
    ChunkedMultiLineStringArray,
    ChunkedMultiPolygonArray,
)

@overload
def chaikin_smoothing(
    input: ChaikinSmoothingT, n_iterations: int
) -> ChaikinSmoothingT: ...
@overload
def chaikin_smoothing(
    input: ArrowArrayExportable, n_iterations: int
) -> LineStringArray | PolygonArray | MultiLineStringArray | MultiPolygonArray: ...
@overload
def chaikin_smoothing(
    input: ArrowStreamExportable, n_iterations: int
) -> (
    ChunkedLineStringArray
    | ChunkedPolygonArray
    | ChunkedMultiLineStringArray
    | ChunkedMultiPolygonArray
): ...
def chaikin_smoothing(
    input: ChaikinSmoothingT | ArrowArrayExportable | ArrowStreamExportable,
    n_iterations: int,
) -> (
    LineStringArray
    | PolygonArray
    | MultiLineStringArray
    | MultiPolygonArray
    | ChunkedLineStringArray
    | ChunkedPolygonArray
    | ChunkedMultiLineStringArray
    | ChunkedMultiPolygonArray
): ...
@overload
def convex_hull(input: ArrowArrayExportable) -> PolygonArray: ...
@overload
def convex_hull(input: ArrowStreamExportable) -> ChunkedPolygonArray: ...
def convex_hull(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> PolygonArray | ChunkedPolygonArray: ...
def densify(
    input: ArrowArrayExportable, max_distance: float
) -> LineStringArray | PolygonArray | MultiLineStringArray | MultiPolygonArray: ...
@overload
def envelope(input: ArrowArrayExportable) -> RectArray: ...
@overload
def envelope(input: ArrowStreamExportable) -> ChunkedRectArray: ...
def envelope(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> RectArray | ChunkedRectArray: ...
@overload
def frechet_distance(
    input: ArrowArrayExportable,
    other: BroadcastGeometry,
) -> Float64Array: ...
@overload
def frechet_distance(
    input: ArrowStreamExportable,
    other: BroadcastGeometry,
) -> ChunkedFloat64Array: ...
def frechet_distance(
    input: ArrowArrayExportable | ArrowStreamExportable,
    other: BroadcastGeometry,
) -> Float64Array | ChunkedFloat64Array: ...
def geodesic_perimeter(input: ArrowArrayExportable) -> Float64Array: ...
@overload
def is_empty(input: ArrowArrayExportable) -> BooleanArray: ...
@overload
def is_empty(input: ArrowStreamExportable) -> ChunkedBooleanArray: ...
def is_empty(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> BooleanArray | ChunkedBooleanArray: ...
@overload
def length(
    input: ArrowArrayExportable,
    *,
    method: LengthMethod | LengthMethodT = LengthMethod.Euclidean,
) -> Float64Array: ...
@overload
def length(
    input: ArrowStreamExportable,
    *,
    method: LengthMethod | LengthMethodT = LengthMethod.Euclidean,
) -> ChunkedFloat64Array: ...
def length(
    input: ArrowArrayExportable | ArrowStreamExportable,
    *,
    method: LengthMethod | LengthMethodT = LengthMethod.Euclidean,
) -> Float64Array | ChunkedFloat64Array: ...
@overload
def line_interpolate_point(
    input: ArrowArrayExportable,
    fraction: float | int | ArrowArrayExportable | NumpyArrayProtocolf64,
) -> PointArray: ...
@overload
def line_interpolate_point(
    input: ArrowStreamExportable,
    fraction: float | int | ArrowStreamExportable,
) -> ChunkedPointArray: ...
def line_interpolate_point(
    input: ArrowArrayExportable | ArrowStreamExportable,
    fraction: float
    | int
    | ArrowArrayExportable
    | ArrowStreamExportable
    | NumpyArrayProtocolf64,
) -> PointArray | ChunkedPointArray: ...
@overload
def line_locate_point(
    input: ArrowArrayExportable, point: GeoInterfaceProtocol | ArrowArrayExportable
) -> Float64Array: ...
@overload
def line_locate_point(
    input: ArrowStreamExportable, point: GeoInterfaceProtocol | ArrowStreamExportable
) -> ChunkedFloat64Array: ...
def line_locate_point(
    input: ArrowArrayExportable | ArrowStreamExportable,
    point: GeoInterfaceProtocol | ArrowArrayExportable | ArrowStreamExportable,
) -> Float64Array | ChunkedFloat64Array: ...
@overload
def polylabel(
    input: ArrowArrayExportable,
    tolerance: float,
) -> PointArray: ...
@overload
def polylabel(
    input: ArrowStreamExportable,
    tolerance: float,
) -> ChunkedPointArray: ...
def polylabel(
    input: ArrowArrayExportable | ArrowStreamExportable,
    tolerance: float,
) -> PointArray | ChunkedPointArray: ...
@overload
def signed_area(
    input: ArrowArrayExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> Float64Array: ...
@overload
def signed_area(
    input: ArrowStreamExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> ChunkedFloat64Array: ...
def signed_area(
    input: ArrowArrayExportable | ArrowStreamExportable,
    *,
    method: AreaMethod | AreaMethodT = AreaMethod.Euclidean,
) -> Float64Array | ChunkedFloat64Array: ...
@overload
def simplify(
    input: SimplifyInputT,
    epsilon: float,
    *,
    method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
) -> SimplifyInputT: ...
@overload
def simplify(
    input: ArrowArrayExportable,
    epsilon: float,
    *,
    method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
) -> (
    PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
): ...
@overload
def simplify(
    input: ArrowStreamExportable,
    epsilon: float,
    *,
    method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
) -> (
    ChunkedPointArray
    | ChunkedLineStringArray
    | ChunkedPolygonArray
    | ChunkedMultiPointArray
    | ChunkedMultiLineStringArray
    | ChunkedMultiPolygonArray
): ...
def simplify(
    input: SimplifyInputT | ArrowArrayExportable | ArrowStreamExportable,
    epsilon: float,
    *,
    method: SimplifyMethod | SimplifyMethodT = SimplifyMethod.RDP,
) -> (
    SimplifyInputT
    | PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
    | ChunkedPointArray
    | ChunkedLineStringArray
    | ChunkedPolygonArray
    | ChunkedMultiPointArray
    | ChunkedMultiLineStringArray
    | ChunkedMultiPolygonArray
): ...
def total_bounds(
    input: ArrowArrayExportable | ArrowStreamExportable,
) -> Tuple[float, float, float, float]: ...

# Top-level table functions

def explode(input: ArrowStreamExportable) -> GeoTable: ...

# I/O

class ParquetFile:
    def __init__(self, path: str, fs: ObjectStore) -> None: ...
    @property
    def num_rows(self) -> int: ...
    @property
    def num_row_groups(self) -> int: ...
    def row_group_bounds(
        self,
        minx_path: Sequence[str],
        miny_path: Sequence[str],
        maxx_path: Sequence[str],
        maxy_path: Sequence[str],
        row_group_idx: int,
    ) -> Tuple[float, float, float, float]: ...
    def row_groups_bounds(
        self,
        minx_path: Sequence[str],
        miny_path: Sequence[str],
        maxx_path: Sequence[str],
        maxy_path: Sequence[str],
    ) -> PolygonArray: ...
    def file_bbox(self) -> List[float] | None: ...
    async def read_async(
        self,
        *,
        batch_size: int | None = None,
        limit: int | None = None,
        offset: int | None = None,
        bbox: Sequence[IntFloat] | None = None,
        bbox_paths: BboxPaths | None = None,
    ) -> GeoTable: ...
    def read(
        self,
        *,
        batch_size: int | None = None,
        limit: int | None = None,
        offset: int | None = None,
        bbox: Sequence[IntFloat] | None = None,
        bbox_paths: BboxPaths | None = None,
    ) -> GeoTable: ...
    async def read_row_groups_async(self, row_groups: Sequence[int]) -> GeoTable: ...
    def read_row_groups(self, row_groups: Sequence[int]) -> GeoTable: ...

class ParquetDataset:
    def __init__(self, paths: Sequence[str], fs: ObjectStore) -> None: ...
    @property
    def num_rows(self) -> int: ...
    @property
    def num_row_groups(self) -> int: ...
    async def read_async(
        self,
        *,
        batch_size: int | None = None,
        limit: int | None = None,
        offset: int | None = None,
        bbox: Sequence[IntFloat] | None = None,
        bbox_paths: BboxPaths | None = None,
    ) -> GeoTable: ...
    def read(
        self,
        *,
        batch_size: int | None = None,
        limit: int | None = None,
        offset: int | None = None,
        bbox: Sequence[IntFloat] | None = None,
        bbox_paths: BboxPaths | None = None,
    ) -> GeoTable: ...

class ParquetWriter:
    def __init__(
        self, file: str | Path | BinaryIO, schema: ArrowSchemaExportable
    ) -> None: ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...
    def close(self) -> None: ...
    def is_closed(self) -> bool: ...
    def write_batch(self, batch: ArrowArrayExportable) -> None: ...
    def write_table(self, table: ArrowStreamExportable) -> None: ...

class ObjectStore:
    def __init__(self, root: str, options: Optional[Dict[str, str]] = None) -> None: ...

def read_csv(
    file: str | Path | BinaryIO,
    geometry_column_name: str,
    *,
    batch_size: int = 65536,
) -> GeoTable: ...
def read_flatgeobuf(
    file: Union[str, Path, BinaryIO],
    *,
    fs: Optional[ObjectStore] = None,
    batch_size: int = 65536,
    bbox: Tuple[float, float, float, float] | None = None,
) -> GeoTable: ...
async def read_flatgeobuf_async(
    path: str,
    *,
    fs: Optional[ObjectStore] = None,
    batch_size: int = 65536,
    bbox: Tuple[float, float, float, float] | None = None,
) -> GeoTable: ...
def read_geojson(
    file: Union[str, Path, BinaryIO], *, batch_size: int = 65536
) -> GeoTable: ...
def read_geojson_lines(
    file: Union[str, Path, BinaryIO], *, batch_size: int = 65536
) -> GeoTable: ...
def read_ipc(file: Union[str, Path, BinaryIO]) -> GeoTable: ...
def read_ipc_stream(file: Union[str, Path, BinaryIO]) -> GeoTable: ...
def read_parquet(
    path: str, *, fs: Optional[ObjectStore] = None, batch_size: int = 65536
) -> GeoTable: ...
async def read_parquet_async(
    path: str, *, fs: Optional[ObjectStore] = None, batch_size: int = 65536
) -> GeoTable: ...
def read_postgis(connection_url: str, sql: str) -> Optional[GeoTable]: ...
async def read_postgis_async(connection_url: str, sql: str) -> Optional[GeoTable]: ...
def read_pyogrio(
    path_or_buffer: Path | str | bytes,
    /,
    layer: int | str | None = None,
    encoding: str | None = None,
    columns: Sequence[str] | None = None,
    read_geometry: bool = True,
    skip_features: int = 0,
    max_features: int | None = None,
    where: str | None = None,
    bbox: Tuple[float, float, float, float] | Sequence[float] | None = None,
    mask=None,
    fids=None,
    sql: str | None = None,
    sql_dialect: str | None = None,
    return_fids=False,
    batch_size=65536,
    **kwargs,
) -> GeoTable: ...
def write_csv(table: ArrowStreamExportable, file: str | Path | BinaryIO) -> None: ...
def write_flatgeobuf(
    table: ArrowStreamExportable,
    file: str | Path | BinaryIO,
    *,
    write_index: bool = True,
) -> None: ...
def write_geojson(
    table: ArrowStreamExportable, file: Union[str, Path, BinaryIO]
) -> None: ...
def write_geojson_lines(
    table: ArrowStreamExportable, file: Union[str, Path, BinaryIO]
) -> None: ...
def write_ipc(
    table: ArrowStreamExportable, file: Union[str, Path, BinaryIO]
) -> None: ...
def write_ipc_stream(
    table: ArrowStreamExportable, file: Union[str, Path, BinaryIO]
) -> None: ...
def write_parquet(
    table: ArrowStreamExportable,
    file: str,
    *,
    encoding: GeoParquetEncoding | GeoParquetEncodingT = GeoParquetEncoding.WKB,
) -> None: ...

# Interop
def from_ewkb(
    input: ArrowArrayExportable,
) -> (
    PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
    | MixedGeometryArray
    | GeometryCollectionArray
): ...
def from_geopandas(input: gpd.GeoDataFrame) -> GeoTable: ...
def from_shapely(
    input,
) -> (
    PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
    | MixedGeometryArray
): ...
def from_wkb(
    input: ArrowArrayExportable,
) -> (
    PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
    | MixedGeometryArray
    | GeometryCollectionArray
): ...
def from_wkt(
    input: ArrowArrayExportable,
) -> (
    PointArray
    | LineStringArray
    | PolygonArray
    | MultiPointArray
    | MultiLineStringArray
    | MultiPolygonArray
    | MixedGeometryArray
    | GeometryCollectionArray
): ...
def to_geopandas(input: ArrowStreamExportable) -> gpd.GeoDataFrame: ...
def to_shapely(input: ArrowArrayExportable) -> NDArray[np.object_]: ...
def to_wkb(input: ArrowArrayExportable) -> WKBArray: ...



import geopandas as gpd
import pyarrow.parquet as pq
import shapely
from geoarrow.rust.core import write_parquet, from_geopandas


def test_write_native_points():
    points = shapely.points([1, 2, 3], [4, 5, 6])
    gdf = gpd.GeoDataFrame({"col1": ["a", "b", "c"]}, geometry=points, crs="EPSG:4326")
    table = from_geopandas(gdf)
    write_parquet(table, "test.parquet", encoding="native")

    schema = pq.read_schema("test.parquet")
    assert (
        schema.field("geometry").metadata[b"ARROW:extension:name"] == b"geoarrow.point"
    )

    # TODO: assert same CRS
    # meta = pq.read_metadata("test.parquet").metadata[b'geo']
    # meta = json.loads(meta)


def test_write_native_multi_points():
    points = shapely.points([1, 2, 3], [4, 5, 6])
    multi_points = shapely.multipoints(
        [
            points[0],
            points[1],
            points[1],
            points[2],
            points[2],
            points[0],
        ],
        indices=[0, 0, 1, 1, 2, 2],
    )

    gdf = gpd.GeoDataFrame(
        {"col1": ["a", "b", "c"]}, geometry=multi_points, crs="EPSG:4326"
    )
    table = from_geopandas(gdf)
    write_parquet(table, "test.parquet", encoding="native")

    schema = pq.read_schema("test.parquet")
    assert (
        schema.field("geometry").metadata[b"ARROW:extension:name"]
        == b"geoarrow.multipoint"
    )



from pathlib import Path


def get_repo_root() -> Path:
    current_dir = Path(".").absolute()

    while current_dir.stem != "geoarrow-rs":
        if current_dir == Path("/"):
            raise ValueError("Could not find repo root; is it named geoarrow-rs?")
        current_dir = current_dir.parent

    return current_dir


REPO_ROOT = get_repo_root()
FIXTURES_DIR = REPO_ROOT / "fixtures"




from io import BytesIO

import geoarrow.rust.core as gars
import geopandas as gpd
import pytest
from geopandas.testing import assert_geodataframe_equal

from tests.utils import FIXTURES_DIR


def test_read_flatgeobuf():
    path = FIXTURES_DIR / "flatgeobuf" / "countries.fgb"
    table = gars.read_flatgeobuf(path)
    assert len(table) == 179
    assert isinstance(table.geometry, gars.ChunkedMultiPolygonArray)


def test_read_flatgeobuf_file_object():
    path = FIXTURES_DIR / "flatgeobuf" / "countries.fgb"
    with open(path, "rb") as f:
        table = gars.read_flatgeobuf(f)
    assert len(table) == 179
    assert isinstance(table.geometry, gars.ChunkedMultiPolygonArray)


def test_round_trip_flatgeobuf():
    path = FIXTURES_DIR / "flatgeobuf" / "countries.fgb"
    table = gars.read_flatgeobuf(path)

    buf = BytesIO()
    gars.write_flatgeobuf(table, buf)
    buf.seek(0)
    table_back = gars.read_flatgeobuf(buf)
    assert table == table_back


@pytest.mark.xfail(reason="fix propagate CRS")
def test_matches_pyogrio():
    path = FIXTURES_DIR / "flatgeobuf" / "countries.fgb"
    table = gars.read_flatgeobuf(path)

    gdf_direct = gpd.read_file(path)
    gdf_from_rust = table.to_geopandas()
    assert_geodataframe_equal(gdf_direct, gdf_from_rust)



import geoarrow.rust.core as gars
import geodatasets
import geopandas as gpd
import numpy as np

nybb_path = geodatasets.get_path("nybb")


def test_read_pyogrio():
    table = gars.read_pyogrio(nybb_path)
    gdf = gpd.read_file(nybb_path)
    assert np.array_equal(table.geometry.to_shapely(), gdf.geometry)



from __future__ import annotations

from typing import Literal, Protocol, Sequence, Tuple, TypeVar, TypedDict, Union
from ._rust import (
    Point,
    LineString,
    Polygon,
    MultiPoint,
    MultiLineString,
    MultiPolygon,
    Geometry,
    GeometryCollection,
    Rect,
    PointArray,
    LineStringArray,
    PolygonArray,
    MultiPointArray,
    MultiLineStringArray,
    MultiPolygonArray,
    MixedGeometryArray,
    GeometryCollectionArray,
    ChunkedPointArray,
    ChunkedLineStringArray,
    ChunkedPolygonArray,
    ChunkedMultiPointArray,
    ChunkedMultiLineStringArray,
    ChunkedMultiPolygonArray,
    ChunkedMixedGeometryArray,
    ChunkedGeometryCollectionArray,
)

try:
    import numpy as np
    from numpy.typing import NDArray

    ScalarType_co = TypeVar("ScalarType_co", bound=np.generic, covariant=True)

except ImportError:
    ScalarType_co = TypeVar("ScalarType_co", covariant=True)


IntFloat = Union[int, float]

AffineInputT = TypeVar(
    "AffineInputT",
    PointArray,
    LineStringArray,
    PolygonArray,
    MultiPointArray,
    MultiLineStringArray,
    MultiPolygonArray,
    MixedGeometryArray,
    GeometryCollectionArray,
    ChunkedPointArray,
    ChunkedMultiPointArray,
    ChunkedLineStringArray,
    ChunkedPolygonArray,
    ChunkedMultiPointArray,
    ChunkedMultiLineStringArray,
    ChunkedMultiPolygonArray,
    ChunkedMixedGeometryArray,
    ChunkedGeometryCollectionArray,
)
"""
Known geoarrow-rust types for input into
[`affine_transform`][geoarrow.rust.core.affine_transform].
"""

AffineTransform = Union[
    Tuple[IntFloat, IntFloat, IntFloat, IntFloat, IntFloat, IntFloat],
    Tuple[
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
        IntFloat,
    ],
    Tuple[IntFloat, ...],
]

AreaMethodT = Literal["ellipsoidal", "euclidean", "spherical"]
"""Acceptable strings to be passed into the `method` parameter for
[`area`][geoarrow.rust.core.area] and
[`signed_area`][geoarrow.rust.core.signed_area].
"""

GeoParquetEncodingT = Literal["wkb", "native"]
"""Acceptable strings to be passed into the `encoding` parameter for
[`write_parquet`][geoarrow.rust.core.write_parquet].
"""

NativeGeometryArrayT = Union[
    PointArray,
    LineStringArray,
    PolygonArray,
    MultiPointArray,
    MultiLineStringArray,
    MultiPolygonArray,
    MixedGeometryArray,
    GeometryCollectionArray,
]

NativeChunkedGeometryArrayT = Union[
    ChunkedPointArray,
    ChunkedLineStringArray,
    ChunkedPolygonArray,
    ChunkedMultiPointArray,
    ChunkedMultiLineStringArray,
    ChunkedMultiPolygonArray,
    ChunkedMixedGeometryArray,
    ChunkedGeometryCollectionArray,
]

LengthMethodT = Literal["ellipsoidal", "euclidean", "haversine", "vincenty"]
"""Acceptable strings to be passed into the `method` parameter for
[`length`][geoarrow.rust.core.length].
"""

SimplifyMethodT = Literal["rdp", "vw", "vw_preserve"]
"""Acceptable strings to be passed into the `method` parameter for
[`simplify`][geoarrow.rust.core.simplify].
"""

SimplifyInputT = TypeVar(
    "SimplifyInputT",
    PointArray,
    LineStringArray,
    PolygonArray,
    MultiPointArray,
    MultiLineStringArray,
    MultiPolygonArray,
    ChunkedMultiPointArray,
    ChunkedLineStringArray,
    ChunkedPolygonArray,
    ChunkedMultiPointArray,
    ChunkedMultiLineStringArray,
    ChunkedMultiPolygonArray,
)
"""Known geoarrow-rust types for input into [`simplify`][geoarrow.rust.core.simplify].
"""


class ArrowSchemaExportable(Protocol):
    """An Arrow or GeoArrow schema or field."""

    def __arrow_c_schema__(self) -> object: ...


class ArrowArrayExportable(Protocol):
    """An Arrow or GeoArrow array or RecordBatch."""

    def __arrow_c_array__(
        self, requested_schema: object | None = None
    ) -> Tuple[object, object]: ...


class ArrowStreamExportable(Protocol):
    """An Arrow or GeoArrow ChunkedArray or Table."""

    def __arrow_c_stream__(self, requested_schema: object | None = None) -> object: ...


class GeoInterfaceProtocol(Protocol):
    """A scalar geometry that implements the Geo Interface protocol."""

    @property
    def __geo_interface__(self) -> dict: ...


class NumpyArrayProtocolf64(Protocol):
    """An object that implements the numpy __array__ method."""

    def __array__(self) -> NDArray[np.float64]: ...


ScalarGeometry = Union[
    GeoInterfaceProtocol,
    Point,
    LineString,
    Polygon,
    MultiPoint,
    MultiLineString,
    MultiPolygon,
    Geometry,
    GeometryCollection,
    Rect,
]

BroadcastGeometry = Union[
    ScalarGeometry,
    NativeGeometryArrayT,
    NativeChunkedGeometryArrayT,
    ArrowArrayExportable,
    ArrowStreamExportable,
]


class BboxPaths(TypedDict):
    minx_path: Sequence[str]
    miny_path: Sequence[str]
    maxx_path: Sequence[str]
    maxy_path: Sequence[str]



# import geoarrow.rust.core as gars

# import geopandas as gpd
# import pandas as pd
# from geoarrow.rust.core import GeoTable

# gdf = gpd.read_file(gpd.datasets.get_path('nybb'))

# table = GeoTable.from_geopandas(gdf)
# exploded_table = table.explode()
# round_trip_gdf = exploded_table.to_geopandas()
# geopandas_impl = gdf.explode(index_parts=False)

# all(round_trip_gdf.geometry.reset_index(drop=True) == geopandas_impl.geometry.reset_index(drop=True))
# # True! Yay!

# ###
# # Perf testing
# large = pd.concat([gdf] * 1000)

# table = GeoTable.from_geopandas(large)
# %timeit exploded_table = table.explode()
# # 590 µs ± 33.2 µs per loop (mean ± std. dev. of 7 runs, 1,000 loops each)

# %timeit gdf_exp = large.explode(index_parts=False)
# # 262 ms ± 10.9 ms per loop (mean ± std. dev. of 7 runs, 1 loop each)



import geoarrow.rust.core as gars
import geodatasets
import geopandas as gpd
import pytest
from geopandas.testing import assert_geodataframe_equal

nybb_path = geodatasets.get_path("nybb")


# Test it works without error
# Superseded by full assertion when CRS is fixed
def test_geopandas_round_trip():
    gdf = gpd.read_file(nybb_path)
    assert isinstance(gdf, gpd.GeoDataFrame)
    table = gars.from_geopandas(gdf)
    _ = table.to_geopandas()


@pytest.mark.xfail(reason="CRS is lost.")
def test_geopandas_round_trip_maintains_crs():
    gdf = gpd.read_file(nybb_path)
    assert isinstance(gdf, gpd.GeoDataFrame)
    table = gars.from_geopandas(gdf)
    gdf_back = table.to_geopandas()
    assert_geodataframe_equal(gdf, gdf_back)
